package be.iminds.ilabt.jfed.experimenter_gui.preferences.subsections;

import be.iminds.ilabt.jfed.experimenter_gui.preferences.AbstractPreferencesSubPane;
import be.iminds.ilabt.jfed.experimenter_gui.util.ui.TextFieldWithStatus;
import be.iminds.ilabt.jfed.git.SingleSshGitAuthPreferences;
import be.iminds.ilabt.jfed.git.SingleUserPassGitAuthPreferences;
import be.iminds.ilabt.jfed.lowlevel.GeniUserProvider;
import be.iminds.ilabt.jfed.preferences.CorePreferenceKey;
import be.iminds.ilabt.jfed.preferences.GuiPreferenceKey;
import be.iminds.ilabt.jfed.preferences.JFedGuiPreferences;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.ui.javafx.util.InformationLabel;
import be.iminds.ilabt.jfed.util.IOUtils;
import be.iminds.ilabt.jfed.util.KeyUtil;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.RadioButton;
import javafx.scene.control.TextArea;
import javafx.scene.layout.BorderPane;
import javafx.stage.FileChooser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;

public class GitAuthenticationPane extends AbstractPreferencesSubPane {
    private final static Logger LOG = LoggerFactory.getLogger(GitAuthenticationPane.class);

    protected final GeniUserProvider geniUserProvider;
    protected final JFedGuiPreferences jFedPreferences;

    @FXML protected RadioButton noneMethodRadioButton;
    @FXML protected RadioButton loginMethodRadioButton;
    @FXML protected RadioButton customSshMethodRadioButton;
    @FXML protected RadioButton userpassMethodRadioButton;
    @FXML protected RadioButton complexMethodRadioButton;

    @FXML protected Label privateKeyLabel;
    @FXML protected TextFieldWithStatus fileTextField;
    @FXML protected Button browseButton;

    @FXML private BorderPane root;

    @Override
    public Node getRoot() {
        return root;
    }
    @Inject
    GitAuthenticationPane(GeniUserProvider geniUserProvider, JFedGuiPreferences jFedPreferences) {
        super("Git Authentication", false);
        this.geniUserProvider = geniUserProvider;
        this.jFedPreferences = jFedPreferences;
    }

    @FXML
    protected void initialize() {
        File privateKeyFile = jFedPreferences.getFile(CorePreferenceKey.PREF_GIT_AUTH_KEYFILE);
        if (privateKeyFile != null)
            fileTextField.setText(privateKeyFile.getAbsolutePath());
        else
            fileTextField.setText("");

        String method = jFedPreferences.getString(CorePreferenceKey.PREF_GIT_AUTH_METHOD);
        if (method == null) method = "NONE";
        switch (method) {
            default:
            case "NONE": { noneMethodRadioButton.setSelected(true); break; }
            case "SSH_LOGIN": { loginMethodRadioButton.setSelected(true); break; }
            case "SSH_CUSTOM": { customSshMethodRadioButton.setSelected(true); break; }
            case "USER_PASS": { userpassMethodRadioButton.setSelected(true); break; }
        }

//        testPrivateKey();
    }

    @FXML
    protected void onBrowsePrivateKeyButtonAction(ActionEvent actionEvent) {
        FileChooser fc = new FileChooser();
        fc.setTitle("Select your Private SSH Key");
//        if (getLoggedInUserPrivateKeyFile() != null)
//            fc.setInitialDirectory(getLoggedInUserPrivateKeyFile().getParentFile());

        File selectedFile = fc.showOpenDialog(fileTextField.getScene().getWindow());
        if (selectedFile != null) {
            fileTextField.setText(selectedFile.getAbsolutePath());

            testPrivateKey();
        }

    }

    protected boolean testPrivateKey() {
        if (fileTextField.getText().length() == 0) {
            fileTextField.setStatus(TextFieldWithStatus.Status.NONE);
            return false;
        }

        try {
            if (testPrivateKeyFile(new File(fileTextField.getText()))) {
                fileTextField.setStatus(TextFieldWithStatus.Status.OK);
                return true;
            } else {
                JFDialogs.create()
                        .owner(root)
                        .message(" Please select a valid private key.")
                        .masthead("The provided key is not valid.")
                        .title("Invalid key file")
                        .showWarning();
                fileTextField.setStatus(TextFieldWithStatus.Status.ERROR);
                return false;
            }
        } catch (IOException ex) {
            JFDialogs.create()
                    .owner(root)
                    .message("An error occured while processing the key file")
                    .masthead("Error")
                    .title("Error")
                    .showException(ex);

            fileTextField.setStatus(TextFieldWithStatus.Status.ERROR);
            return false;
        }
    }

    protected boolean testPrivateKeyFile(File file) throws IOException {
        if (file.exists() && file.isFile()) {
            //check if it is an putty private key
            BufferedReader reader = new BufferedReader(new FileReader(file));
            StringBuilder fileContents = new StringBuilder();

            String line;
            while ((line = reader.readLine()) != null) {
                fileContents.append(line);
            }

            reader.close();


            return KeyUtil.hasAnyPrivateKey(fileContents.toString());
        } else
            return false;
    }

    @Override
    public boolean check() {
        if (customSshMethodRadioButton.isSelected()) {
            if (!testPrivateKey())
                return false;
        }

        return true;
    }

    @Override
    public boolean save() {
        if (customSshMethodRadioButton.isSelected()) {
            jFedPreferences.setString(CorePreferenceKey.PREF_GIT_AUTH_METHOD, "SSH_CUSTOM");
            File privKeyFile = new File(fileTextField.getText());
            jFedPreferences.setFile(CorePreferenceKey.PREF_GIT_AUTH_KEYFILE, privKeyFile);
            return true;
        } else {
            jFedPreferences.clear(CorePreferenceKey.PREF_GIT_AUTH_KEYFILE);
        }

        if (noneMethodRadioButton.isSelected()) {
            jFedPreferences.setString(CorePreferenceKey.PREF_GIT_AUTH_METHOD, "NONE");
            return true;
        }
        if (loginMethodRadioButton.isSelected()) {
            jFedPreferences.setString(CorePreferenceKey.PREF_GIT_AUTH_METHOD, "SSH_LOGIN");
            return true;
        }
        if (userpassMethodRadioButton.isSelected()) {
            jFedPreferences.setString(CorePreferenceKey.PREF_GIT_AUTH_METHOD, "NONE");
//            jFedPreferences.setString(CorePreferenceKey.PREF_GIT_AUTH_METHOD, "USER_PASS");
//            throw new RuntimeException("Not yet supported");
            return true;
        }

        return false;
    }
}
